<?php
namespace Teknisa\EST\Modules\Ajuste\Services;

use Teknisa\EST\Modules\Ajuste\Factories\AjusteEstoque as AjusteFactory;

class AjusteEstoque {

    protected $AjusteFactory;
    protected $nrorg;
    protected $filialLogada;
    protected $operador;
    protected $newCode;

    public function __construct (AjusteFactory $AjusteEstoque, $environmment, $newCode) {
        $this->AjusteFactory = $AjusteEstoque;
        $this->nrorg = $environmment->getNrorg();
        $this->filialLogada = $environmment->getCdFilial();
        $this->operador = $environmment->getCdOperador();
        $this->newCode = $newCode;
    }

    public function changeIsNew ($adjustment) {
        $adjustment['__is_new'] = false;
        foreach ($adjustment['PRODUCTS'] as &$product) {
            $product['__is_new'] = false;
        }

        return $adjustment;
    }

    public function checkRelationship ($nrlancestq, $products) {
        foreach ($products as &$product) {
            if ($product['NRLANCESTQ'] != $nrlancestq) {
                throw new \Exception("O produto ".$product['NMPROUTO']." do ajuste de sequencial "."$nrlancestq"." não tem o mesmo sequencial do seu ajuste.", 1);                
            }
        }
    }

    public function containsProducts ($productsArray) {
        if (count($productsArray) <= 0) {
            throw new \Exception("Erro. Um dos ajustes está sem produto", 1);
        }
    }

    public function generatedValidSequential () {
        $filial = $this->filialLogada;
        return $this->newCode->getCodeByTableName('LANCTOEST'.$filial, $this->nrorg);
    }

    public function getParams($cdfilial){
        return $this->AjusteFactory->getParams($cdfilial);
    }
    
    public function saveAdjustment ($adjustment) {
        if(!isset($adjustment['CDALMOXARIFE']))
            $adjustment['CDALMOXARIFE'] = ' ';
        if(!isset($adjustment['CDLOCALESTOQ']))
            $adjustment['CDLOCALESTOQ'] = ' ';
        
        $parameters = array(
            "NRLANCESTQ" => $adjustment['NRLANCESTQ'],
            "DTLANCESTQ" => $adjustment['DTLANCESTQ'],
            "DSLANCESTQ" => $adjustment['DSLANCESTQ'],
            "CDFILIAL" => $adjustment['CDFILIAL'],
            "CDFILIMOVI" => $adjustment['CDFILIMOVI'],
            "IDTPLANCTO" => $adjustment['IDTPLANCTO'],
            "IDALTCUSMED" => $adjustment['IDALTCUSMED'],
            "IDSTATUSCTBEST" => $adjustment['IDSTATUSCTBEST'],
            "CDALMOXARIFE" => $adjustment['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $adjustment['CDLOCALESTOQ'],
            "CDOPERADOR"=> $adjustment['CDOPERADOR'],
            "NRORG" => $adjustment['NRORG']
        );
        $this->AjusteFactory->insertLanctoestoq($parameters);
    }

    public function saveProducts ($product) {
        if(!isset($product['IDAJUSTEIMPORT']))
            $product['IDAJUSTEIMPORT'] = 'N';
        $product['IDAJUSTEIMPORT'] = $product['IDAJUSTEIMPORT'] == 'OK'? 'S' : 'N';
        
        $parameters = array(
            "NRLANCESTQ" => $product['NRLANCESTQ'],
            "NRSEQUITEM" => $product['NRSEQUITEM'],
            "CDPRODUTO" => $product['CDPRODUTO'],
            "NRLOTEESTQ" => $product['NRLOTEESTQ'],
            "NRSUBLOTE" => $product['NRSUBLOTE'],
            "CDALMOXARIFE" => $product['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $product['CDLOCALESTOQ'],
            "CDFILIAL" => $product['CDFILIAL'],
            "CDFILIMOVI" => $product['CDFILIMOVI'],
            "QTLANCTOEST" => $product['QTLANCTOEST'],
            "IDEVOLESTOQ" => $product['IDEVOLESTOQ'],
            "IDTIPOMOVI" => is_null($product['IDTIPOMOVI'])? '1' : $product['IDTIPOMOVI'],
            "VRLANCTOBRUT" => $product['VRLANCTOBRUT'],
            "CDPRODMOVI" => $product['CDPRODMOVI'],
            "DTLANCMOVI" => $product['DTLANCMOVI'],
            "QTTOTLANCTO" => $product['QTTOTLANCTO'],
            "CDOPERADOR" => $product['CDOPERADOR'],
            "IDAJUSTEIMPORT" => $product['IDAJUSTEIMPORT'],
            "NRORG" => $product['NRORG']
        );
        $this->AjusteFactory->insertItlanctoest($parameters);
    }

    public function setDefaultColumnsForAdjustment ($adjustment) {
        $adjustment['CDFILIMOVI'] = $adjustment['CDFILIAL']? $adjustment['CDFILIAL'] : $this->filialLogada;
        $adjustment['CDFILIAL'] =  $this->filialLogada;
        $adjustment['IDTPLANCTO'] = '1';
        $adjustment['IDALTCUSMED'] = 'N';
        $adjustment['IDSTATUSCTBEST'] = 'P';
        $adjustment['CDOPERADOR'] = $this->operador;
        $adjustment['NRORG'] = $this->nrorg;
        $adjustment['SITUACAO'] = 'E';

        return $adjustment;
    }

    public function setDefaultColumnsForProducts ($products, $date, $filial) {
        foreach ($products as $key => &$product) {
            //Rotina para revalidar e atualizar vinculo com produto de estoque.
            $prodEsto = !isset($product['CDPRODESTO'])? $this->AjusteFactory->getProdutoEstoque($product['CDPRODUTO']) : $product['CDPRODESTO'];
            
            $product['NRSEQUITEM'] = str_pad($key+1, 4, '0', STR_PAD_LEFT);
            $product['IDTIPOMOVI'] = '1';
            $product['CDFILIMOVI'] = $filial? $filial : $this->filialLogada;
            $product['CDFILIAL'] = $this->filialLogada;
            $product['VRLANCTOBRUT'] = 0;
            $product['IDEVOLESTOQ'] = 'N';
            $product['CDPRODMOVI'] = $prodEsto;
            $product['QTLANCTOEST'] = $product['QTTOTLANCTO'] * $product['VRFATOCONV'];
            $product['DTLANCMOVI'] = $date;
            $product['CDOPERADOR'] = $this->operador;
            $product['NRORG'] = $this->nrorg;

            if (empty($product['NRLOTEESTQ'])) {
                $product['NRLOTEESTQ'] = " ";
            }
            if (empty($product['NRSUBLOTE'])) {
                $product['NRSUBLOTE'] = " ";
            }
            if (empty($product['CDALMOXARIFE'])) {
                $product['CDALMOXARIFE'] = " ";
            }
            if (empty($product['CDLOCALESTOQ'])) {
                $product['CDLOCALESTOQ'] = " ";
            }
        }

        return $products;
    }

    public function setValidSequential ($nrlancestq, $products) {
        foreach ($products as &$product) {
            $product['NRLANCESTQ'] = $nrlancestq;
        }

        return $products;
    }
    
    public function getParamAppAtuEstq($filial){
        try{
            
            $param = $this->AjusteFactory->getParamAppAtuEstq($filial)[0]['IDAPPATUESTQ'];
            if($param == 'S')
                return true;
            else
                return false;
                
        }catch (\Exception $e){
            return false;
        }
    }
    
        
    public function getMaxNrSeqItemAjuste($cdfilial, $nrlancestq) {
        $maxNrSeqItem = $this->AjusteFactory->getMaxNrSeqItemAjuste($cdfilial, $nrlancestq);
        
        $nrsequitem = ($maxNrSeqItem[0]["NRSEQUITEM"] != null) ? $maxNrSeqItem[0]["NRSEQUITEM"] : 1;
        $nrsequitem = str_pad($nrsequitem, 4, "0", STR_PAD_LEFT);
        
        return $nrsequitem;
    }
    
    
    public function consolidadeToStockProducts($products){
        //Função movida para o botão de atualizar estoque, nao existe necessidade de consolidar nesta parte.
      /* $arraySize = count($products);
        for ($a = 0; $a < $arraySize; $a++) {
                for($b = $a; $b < $arraySize; $b++){
                    if( ($products[$b]['NRSEQUITEM'] != $products[$a]['NRSEQUITEM']) && 
                        ($products[$b]['CDPRODESTO'] == $products[$a]['CDPRODESTO']) &&
                        ($products[$b]['NRLOTEESTQ'] == $products[$a]['NRLOTEESTQ']) &&
                        ($products[$b]['CDALMOXARIFE'] == $products[$a]['CDALMOXARIFE']) &&
                        ($products[$b]['CDLOCALESTOQ'] == $products[$a]['CDLOCALESTOQ']) &&
                        ($products[$b]['NRSUBLOTE'] == $products[$a]['NRSUBLOTE']) )
                        {
                         $products[$a]['QTLANCTOEST'] += $products[$b]['QTTOTLANCTO'] * $products[$b]['VRFATOCONV'];
                         $products[$a]['QTTOTLANCTO'] += ($products[$b]['QTTOTLANCTO'] * $products[$b]['VRFATOCONV']) / $products[$a]['VRFATOCONV'];
                         array_splice($products, $b, 1);
                         $b--;
                         $arraySize = $arraySize - 1;
                        }
                }
        }*/
        return $products;
    }

}